#include <stdio.h>
#include <stdlib.h>
#include "../include/util_gettime.h"
#define IND(A, x, y, d) A[(x)*(d)+(y)] 
#define min( a, b ) ( ((a) < (b)) ? (a) : (b) )
#define BLOCK_X 16
#define BLOCK_Y 16
#define BLOCK_Z 16

unsigned getSeed()
{//---------------------
    struct timeval tp;
    int rtn;
    rtn=gettimeofday(&tp, NULL);
    return((signed int)tp.tv_usec);
}


float testMM(const int x, const int y, const int z)
{//-------------------------------------------------
  double *A, *B, *C, *Cx;
  long started, ended;
  float timeTaken;
  int i, j, k, i0, j0, k0;
  A = (double *)calloc(x*y, sizeof(double));
  B = (double *)malloc(sizeof(double)*x*z);
  C = (double *)malloc(sizeof(double)*y*z);
  Cx = (double *)malloc(sizeof(double)*y*z);

  srand(getSeed());
  for (i = 0; i < x*z; i++) B[i] = (double) rand() ;
  for (i = 0; i < y*z; i++) C[i] = (double) rand() ;

  started = example_get_time();

  //transpose C
  for(j =0; j < y; j++)
    for(k=0; k < z; k++)
      IND(Cx,j,k,z) = IND(C, k, j, y); 
  
  //blocked multiplication
  for (i = 0; i < x; i += BLOCK_X) 
    for (j = 0; j < y; j += BLOCK_Y) 
      for (k = 0; k < z; k += BLOCK_Z) 
	for (i0 = i; i0 < min(i + BLOCK_X, x); i0++) 
	  for (j0 = j; j0 < min(j + BLOCK_Y, y); j0++) 
	    for (k0 = k; k0 < min(k + BLOCK_Z, z); k0++)
	      IND(A,i0,j0,y) += IND(B,i0,k0,z) * IND(Cx,j0,k0,z);

  ended = example_get_time();  
  timeTaken = (ended - started)/1.f;

  free(A);
  free(B);
  free(C);
  free(Cx);

  return timeTaken;
}

int main(int argc, char * argv[])
{
  float timeTaken;
  if (argc == 1){
     timeTaken = testMM(2048, 2048, 2048);
     return 0;
  }
  if(argc != 4){
    printf("Usage: %s l m n (dimensions of the two matrices)\n", argv[0]);
    return 0;
  }
  timeTaken = testMM(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]));
  printf("Running time = %5.0f ms\n", timeTaken);
  return 0;
}
