#include <stdio.h>
#include <stdlib.h>
#include "../include/util_gettime.h"
#define IND(A, x, y, d) A[(x)*(d)+(y)] 

unsigned getSeed()
{
  struct timeval tp;
  int rtn;
  rtn=gettimeofday(&tp, NULL);
  return((signed int)tp.tv_usec);
}

float testMM(const int x, const int y, const int z)
{
  double *A, *B, *C, *Cx;
  long started, ended;
  float timeTaken;
  int i, j, k;
  
  A  = (double *)calloc(x*y, sizeof(double)); //initialize to 0
  B  = (double *)malloc(sizeof(double)*x*z);
  C  = (double *)malloc(sizeof(double)*y*z);
  Cx = (double *)malloc(sizeof(double)*z*y);
  
  //generate random data for testing
  srand(getSeed());
  for (i = 0; i < x*z; i++) B[i] = (double) rand() ;
  for (i = 0; i < y*z; i++) C[i] = (double) rand() ;
  
  started = example_get_time();
  
  //transpose C
  for(j =0; j < y; j++)
    for(k=0; k < z; k++)
      IND(Cx,j,k,z) = IND(C, k, j, y); 
  
  //naive multiplication
  for (i = 0; i < x; i++) 
    for (j = 0; j < y; j++) 
      for (k = 0; k < z; k++) 
        //  A[i][j] += B[i][k] * Cx[j][k];
	IND(A, i, j, y) += IND(B, i, k, z)*IND(Cx, j, k, z);

  ended = example_get_time();  
  timeTaken = (ended - started)/1.f;
  
  free(A);
  free(B);
  free(C);
  free(Cx);
  
  return timeTaken;
}

int main(int argc, char * argv[])
{
  float timeTaken;
  if (argc == 1){
     timeTaken = testMM(2048, 2048, 2048);
     return 0;
  }
  if(argc != 4){
    printf("Usage: %s l m n (dimensions of the two matrices)\n", argv[0]);
    return 0;
  }
  timeTaken = testMM(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]));
  printf("Running time = %5.0f ms\n", timeTaken);
  return 0;
}
