
/* To compile: gcc -O3 -o MatTran1 MatTran1.c */
#include <stdio.h>
#include <stdlib.h>
#include "../include/util_gettime.h"

/* Traverse the trapezoidal space (i, j) where
   i0 <= i < i1
   j0 + (i - i0) * dj0 <= j < j1 
   
   Author: Matteo Frigo
*/
void MatTran1(int *A, 
	      int lda, int i0,  int i1,
	      int j0,  int dj0, int j1)
{
    const int THRESHOLD = 4; 

    int di, dj, dj2, di2, i, j, x;

 tail:
    di = i1 - i0; 
    dj = j1 - j0;
    
    if (dj >= 2 * di && dj > THRESHOLD) {
	dj2 = dj / 2;
	MatTran1(A, lda, i0, i1, j0, dj0, j0 + dj2);
	j0 += dj2; dj0 = 0; goto tail;
    } else if (di > THRESHOLD) {
	di2 = di / 2;
	MatTran1(A, lda, i0, i0 + di2, j0, dj0, j1);
	i0 += di2; j0 += dj0 * di2; goto tail;
    } else {
	for (i = i0; i < i1; ++i) {
	    for (j = j0; j < j1; ++j) {
		x = A[j * lda + i];
		A[j * lda + i] = A[i * lda + j];
		A[i * lda + j] = x;
	    }
	    j0 += dj0;
	}
    }
}

/* Print an n-by-n matrix */
void print_matrix(int* a, int n)
{
  int i,j;
  for(i=0;i<n;i++) {
   for( j=0; j<n; j++) {
     printf("%d   ", a[n*i+j]);
     if (j == n-1) printf("\n");
   }
  }
  printf("\n");
}

/* Create a random n-by-n matrix with range 0 to n-1 */
void random_matrix(int* a, int n)
{
  int i,j;
  for(i=0;i<n;i++) {
    for(j=0;j<n;j++) {
      a[i*n + j] = rand()%n;
    }
  }
}


int main(int argc, char * argv[])
{
  int i, j, n, s, *A;
    
  long started, ended;
  float timeTaken;
  
  if(argc != 2){
    printf("Usage: %s n (square matrix size)\n", argv[0]);
    return 0;
  }
  n = atoi(argv[1]);

  s = n * n;
  if (s < 1000000000) {
    A = (int *) malloc(s * sizeof(int));
    random_matrix(A,n);
    
    if (n < 20) {printf("Starting matrix: \n"); print_matrix(A,n);}
    
    started = example_get_time();
    
    MatTran1(A, n, 0, n, 0, 1, n);
    
    ended = example_get_time();  
    timeTaken = (ended - started)/1.f;
    printf("Running time = %5.0f ms\n", timeTaken);
    
    
    if (n < 20) {printf("Transposed matrix: \n"); print_matrix(A,n);}
  }
  
  free(A);
  
  return 0;
}

