#include <stdio.h>
#include <stdlib.h>
#include "../include/util_gettime.h"

/* Transpose naively an n-by-n matrix */
void transpose_matrix(int* a,  int n)
{
  int i,j, tmp;

  for(i=0;i<n;i++) {
    for( j=i+1; j<n; j++) {
      tmp = a[i*n + j];
      a[i*n + j] = a[j*n + i];
      a[j*n + i] = tmp;
      }
  }
}

/* Print an n-by-n matrix */
void print_matrix(int* a,  int n)
{
  int i,j;
  for(i=0;i<n;i++) {
    for( j=0; j<n; j++) {
      printf("%d ", a[n*i+j]);
      if (j == n-1) printf("\n");
    }
  }
  printf("\n");
}

/* Create a random n-by-n matrix */
void random_matrix(int* a,  int n)
{
  int i,j;

  for(i=0;i<n;i++) {
    for(j=0;j<n;j++) {
      a[i*n + j] = rand()%n;
    }
  }
}

int main(int argc, char * argv[]) 
{
  int n, s;
  int* a;
  long started, ended;
  float timeTaken;

  if(argc != 2){
      printf("Usage: %s n (dimension of the square matrix)\n", argv[0]);
      return 0;
   }
  n = atoi(argv[1]);
  s = n * n;
  if (s < 1000000000) {    
    a = (int *) malloc(s * sizeof(int)); 
    random_matrix(a,n);
    if (n < 10) print_matrix(a,n);

    started = example_get_time();

    transpose_matrix(a,n);        //to compare

    ended = example_get_time();  
    timeTaken = (ended - started)/1.f;
    printf("Running time = %5.0f ms\n", timeTaken);

    if (n < 10) print_matrix(a,n);
  }

  free(a);
  return 0;
}
