#include <stdio.h>
#include <stdlib.h>
 
void counting_sort_mm(int *array, int n, int min, int max)
{
  int i, j, z;
 
  int range = max - min + 1;
  int *count = (int *)calloc(range, sizeof(*array));
 
  //for(i = 0; i < range; i++) 
  //  count[i] = 0;
  for(i = 0; i < n; i++) 
    count[ array[i] - min ]++;
 
  for(i = min, z = 0; i <= max; i++) {
    for(j = 0; j < count[i - min]; j++) {
      array[z++] = i;
    }
  } 
 
  free(count);
}
 
//Testing (we suppose the oldest human being is less than 140 years old)

#define N 1000
#define MAX_AGE 140
 
int main()
{//-------------------------
  int ages[N], i;
  
  for(i=0; i < N; i++) 
    ages[i] = rand()%MAX_AGE;

  counting_sort_mm(ages, N, 0, MAX_AGE);

  for(i=0; i < N; i++) {
    printf("%d, ", ages[i]);
  }
  printf("\n");

  return 0;
}

/*
Wiki Note: we know that, given an array of integers, its maximum and minimum values can be always found; but if we imagine the worst case for an array of 32 bit integers, we see that in order to hold the counts, we need an array of 232 elements, i.e., we need, to hold a count value up to 232-1, more or less 4 Gbytes. So the counting sort is more practical when the range is (very) limited and minimum and maximum values are known a priori. (Anyway sparse arrays may limit the impact of the memory usage) 
*/
